/*============================================================================
  File:     BackupIntegrity.sql

  Summary:  This script shows the various options
			for backup integrity checking

			Uses a pre-corrupted database from
			http://www.sqlskills.com/blogs/paul/content/binary/broken2005.zip

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

IF DATABASEPROPERTYEX ('broken', 'Version') > 0
	DROP DATABASE broken;

-- Restore the database
RESTORE DATABASE broken
	FROM DISK='C:\SQLskills\broken2005.bck'
	WITH MOVE 'broken' TO 'C:\SQLskills\broken.mdf',
	MOVE 'broken_log' TO 'C:\SQLskills\broken_log.ldf';
GO

-- And checkout the corruption
SELECT * FROM broken..brokentable;
GO

-- More on this in Module 8...

-- Let's backup the database
BACKUP DATABASE broken
	TO DISK='C:\SQLskills\broken2.bck'
	WITH CHECKSUM;
GO

-- Force it
BACKUP DATABASE broken
	TO DISK='C:\SQLskills\broken2.bck'
	WITH CHECKSUM, CONTINUE_AFTER_ERROR;
GO

-- Check validity of the good backup
RESTORE VERIFYONLY
	FROM DISK='C:\SQLskills\broken2005.bck';
GO

-- And the forced backup...
RESTORE VERIFYONLY
	FROM DISK='C:\SQLskills\broken2.bck';
GO

-- What about using WITH CHECKSUM?
RESTORE VERIFYONLY
	FROM DISK='C:\SQLskills\broken2.bck'
	WITH CHECKSUM;
GO

-- Doesn't matter because we forced it.
-- What about the good backup?
RESTORE VERIFYONLY
	FROM DISK='C:\SQLskills\broken2005.bck'
	WITH CHECKSUM;
GO

-- What about restoring the forced backup?
RESTORE DATABASE broken
	FROM DISK='C:\SQLskills\broken2.bck'
	WITH REPLACE;
GO

-- Force it...
RESTORE DATABASE broken
	FROM DISK='C:\SQLskills\broken2.bck'
	WITH REPLACE, CONTINUE_AFTER_ERROR;
GO
