/*============================================================================
  File:     DatabaseSnapshot.sql

  Summary:  This script shows recovery from
			a database snapshot

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE SalesDB;
GO

-- Try to create the database snaposhot
CREATE DATABASE SalesDB_Snapshot
ON (
	NAME = N'SalesDBData',
	FILENAME = N'C:\SQLskills\test\SalesDBData.mdfss')
AS SNAPSHOT OF SalesDB;
GO

-- Hmm - what files are there?
sp_helpfile;
GO

-- Need one snapshot file per database file
CREATE DATABASE SalesDB_Snapshot ON
	(NAME = N'SalesDBData',
	FILENAME = N'C:\SQLskills\test\SalesDBData.mdfss'),
	(NAME = N'SalesDBSalesDataPartition1',
	FILENAME = N'C:\SQLskills\test\SalesDBSalesDataPartition1.ndfss'),
	(NAME = N'SalesDBSalesDataPartition2',
	FILENAME = N'C:\SQLskills\test\SalesDBSalesDataPartition2.ndfss'),
	(NAME = N'SalesDBSalesDataPartition3',
	FILENAME = N'C:\SQLskills\test\SalesDBSalesDataPartition3.ndfss'),
	(NAME = N'SalesDBSalesDataPartition4',
	FILENAME = N'C:\SQLskills\test\SalesDBSalesDataPartition4.ndfss')
AS SNAPSHOT OF SalesDB;
GO

-- How big are they in Windows Explorer?
-- Checkout 'size on disk'...

-- Checkout the ACTUAL file size rather than the
-- appearance of the file's size!
SELECT v.file_id AS [File ID],
	f.[name] AS [Database File Name],
	v.size_on_disk_bytes / 1024 AS [Sparse File PHYSICAL Size (KB)],
	f.[size] * 8 / 1024 AS [Sparse File LOGICAL Size (MB)],
	f.physical_name AS [Physical File Name]
FROM sys.dm_io_virtual_file_stats (DB_ID ('SalesDB_Snapshot'), -1) AS v
	JOIN sys.master_files AS f ON v.file_id = f.file_id
WHERE f.database_id = v.database_id
	AND f.database_id = DB_ID('SalesDB_Snapshot');
GO

USE SalesDB;
GO

DROP TABLE Sales;
GO

select * from sales

-- Phew - we have a DB Snapshot...

USE SalesDB_Snapshot;
GO

SELECT COUNT (*) FROM Sales;
GO

-- It's all there. Did you notice how
-- fast the table was dropped? Nothing
-- actually got copied this time...

-- Should we set the database to restricted
-- access? Probably!
USE master;
GO

ALTER DATABASE SalesDB
	SET RESTRICTED_USER
	WITH ROLLBACK IMMEDIATE;
GO

-- Now we need to recreate the sales table...

-- Finally, we can recover all of the
-- sales data from an INSERT/SELECT

SET IDENTITY_INSERT
	SalesDB.dbo.Sales ON;
GO

INSERT SalesDB.dbo.Sales
		( SalesID
		, SalesPersonID
		, CustomerID
		, ProductID
		, Quantity)
SELECT *
	FROM SalesDB_Snapshot.dbo.Sales 
	WHERE SalesID < 100000;
GO

SET IDENTITY_INSERT
	SalesDB.dbo.Sales OFF;
GO

ALTER DATABASE SalesDB
	SET MULTI_USER;
GO

-- Or we could just revert to the snapshot
RESTORE DATABASE SalesDB
 FROM DATABASE_SNAPSHOT = 'SalesDB_Snapshot';
GO

SELECT COUNT (*) FROM SalesDB.dbo.Sales;
GO


